#!/usr/bin/env python3
"""
A small pyQt5-Webbrowser to access https://www.evilarena.de/serverlist/

the serverlist website shows connect/start oa/exit buttons when accessed with oa-connect.
please change the openarena (optional), openarena_vars and w, h and zoom variables for your needs below.

Arch Linux Dependencies:
pacman -S python python-pyqt5 python-pyqt5-webengine
Ubuntu Dependencies (should be, didn't try):
python3 python3-pyqt5 python3-pyqt5.qtwebengine
"""
__author__ = "Pan!C"
__copyright__ = "Copyright 2023, Pan!C"
__license__ = "GPLv3"
__version__ = "3.0"
__email__ = "webmaster@evillab.de"
__website__ = "https://evilarena.de/oa-connect/"

#additional commands to pass to openarena, example: +com_hunkmegs 512
openarena_vars = "+com_hunkmegs 512"

#OpenArena executable, optional variable. The script searches in $PATH by default.
#openarena=""

#Window parameters, width x height
w = 1300; h = 600
#Default zoomfactor
zoom = 1.0

# importing required libs
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5 import QtGui
from PyQt5.QtWebEngineWidgets import *
import subprocess
import os
import sys
import re
import shutil

if not 'openarena' in globals():
	openarena = shutil.which("openarena")
	if (openarena is None):
		print("OpenArena executable not found, try to set the optional 'openarena' variable in line 20.")
		sys.exit()

# Creating the main window class
class OAConnectWindow(QMainWindow):

	# Constructor
	def __init__(self, *args, **kwargs):
		super(OAConnectWindow, self).__init__(*args, **kwargs)

		# Creating the QWebEngineView
		self.oa_connect = QWebEngineView()
		self.oa_connect.setZoomFactor(zoom)

		scriptpath = os.path.dirname(os.path.realpath(__file__))
		imagepath = "".join((scriptpath, "/oa-connect.ico"))
		self.setWindowIcon(QtGui.QIcon(imagepath))

		useragent = "".join(("oa-connect v",  __version__))
		self.oa_connect.page().profile().setHttpUserAgent(
			useragent
		)

		# Default url is the evilarena.de serverlist querypage
		self.oa_connect.setUrl(QUrl("https://www.evilarena.de/serverlist/query/?oa-connect-start"))

		# Adding an action when the url get changed
		self.oa_connect.urlChanged.connect(self.oa_connect_check)

		self.setCentralWidget(self.oa_connect)
		self.show()

	# check the updated url for oa-connect references.
	# called by self.oa_connect
	def oa_connect_check(self, q):

		if "https://www.evilarena.de/oa-server-connect/" in q.toString():
			try:
				ip = re.search(r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}:[0-9]+', q.toString() ).group()
				#self.setWindowTitle("oa-connect")
				connectline = " ".join((openarena, "+connect", ip, openarena_vars, "&"))
				print ("start:", connectline)
				subprocess.Popen(connectline, shell=True)
			except AttributeError:
				ip = re.search(r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}:[0-9]+', q.toString() )

		elif "#local-oa-connect" in q.toString():
			connectline = " ".join((openarena, openarena_vars, "&"))
			print ("start:", connectline)
			subprocess.Popen(connectline, shell=True)

		elif "#exit-oa-connect" in q.toString():
			print("Bye.")
			sys.exit()

# creating the pyQt5 app
app = QApplication(sys.argv)
app.setApplicationName("OA-Connect")
win = OAConnectWindow()
win.resize(w, h)
win.show ()
app.exec_()
